@@ -12,7 +12,9 @@ from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.engine import Engine
 from sqlalchemy import event, create_engine, Table
 from sqlalchemy import or_, and_
-from .utils import json_serializer, get_simbad_names
+from . import REFERENCE_TABLES, PRIMARY_TABLE, PRIMARY_TABLE_KEY, FOREIGN_KEY
+from .utils import json_serializer, get_simbad_names, deprecated_alias, datetime_json_parser
+from .spectra import load_spectrum
 
 try:
     from .version import version as __version__
@@ -27,8 +29,7 @@ Base = declarative_base()
 class AstrodbQuery(Query):
     # Subclassing the Query class to add more functionality.
     # See: https://stackoverflow.com/questions/15936111/sqlalchemy-can-you-add-custom-methods-to-the-query-object
-    def astropy(self):
-        # Allow SQLAlchemy query output to be formatted as an astropy Table
+    def _make_astropy(self):
         temp = self.all()
         if len(temp) > 0:
             t = AstropyTable(rows=temp, names=temp[0].keys())
@@ -36,15 +37,86 @@ class AstrodbQuery(Query):
             t = AstropyTable(temp)
         return t
 
-    def table(self):
+    def astropy(self, spectra=None, spectra_format=None, **kwargs):
+        """
+        Allow SQLAlchemy query output to be formatted as an astropy Table
+
+        Parameters
+        ----------
+        spectra : str or list
+            List of columns to process as spectra
+        spectra_format : str
+            Format to apply for all spectra. Default: None means specutils will attempt to find the best one.
+
+        Returns
+        -------
+        t : astropy.Table
+            Table output of query
+        """
+
+        t = self._make_astropy()
+
+        # Apply spectra conversion
+        if spectra is not None:
+            if not isinstance(spectra, (list, tuple)):
+                spectra = [spectra]
+            for col in spectra:
+                if col in t.colnames:
+                    t[col] = [load_spectrum(x, spectra_format=spectra_format) for x in t[col]]
+
+        return t
+
+    def table(self, *args, **kwargs):
         # Alternative for getting astropy Table
-        return self.astropy()
+        return self.astropy(*args, **kwargs)
 
-    def pandas(self):
-        return pd.DataFrame(self.all())
+    def pandas(self, spectra=None, spectra_format=None, **kwargs):
+        """
+        Allow SQLAlchemy query output to be formatted as a pandas DataFrame
+
+        Parameters
+        ----------
+        spectra : str or list
+            List of columns to process as spectra
+        spectra_format : str
+            Format to apply for all spectra. Default: None means specutils will attempt to find the best one.
+
+        Returns
+        -------
+        df : pandas.DataFrame
+            DataFrame output of query
+        """
+
+        df = pd.DataFrame(self.all())
+
+        # Apply spectra conversion
+        if spectra is not None:
+            if not isinstance(spectra, (list, tuple)):
+                spectra = [spectra]
+            for col in spectra:
+                if col in df.columns.to_list():
+                    df[col] = df[col].apply(lambda x: load_spectrum(x, spectra_format=spectra_format))
+
+        return df
+
+    def spectra(self, spectra=['spectrum'], fmt='astropy', **kwargs):
+        """
+        Convenience method fo that uses default column name for spectra conversion
+
+        Parameters
+        ----------
+        spectra : str or list
+            List of columns to process as spectra
+        fmt : str
+            Output format (Default: astropy)
+        """
+        if fmt == 'pandas':
+            return self.pandas(spectra=spectra, **kwargs)
+        else:
+            return self.astropy(spectra=spectra, **kwargs)
 
 
-def load_connection(connection_string, sqlite_foreign=True, base=None):
+def load_connection(connection_string, sqlite_foreign=True, base=None, connection_arguments={}):
     """Return session, base, and engine objects for connecting to the database.
 
     Parameters
@@ -57,6 +129,8 @@ def load_connection(connection_string, sqlite_foreign=True, base=None):
         Flag to enable foreign key checks for SQLite. Default: True
     base : SQLAlchemy base object
         Use an existing base class. Default: None (ie, creates a new one)
+    connection_arguments : dict
+        Additional connection arguments, like {'check_same_thread': False}
 
     Returns
     -------
@@ -69,7 +143,7 @@ def load_connection(connection_string, sqlite_foreign=True, base=None):
         Provides a source of database connectivity and behavior.
     """
 
-    engine = create_engine(connection_string)
+    engine = create_engine(connection_string, connect_args=connection_arguments)
     if not base:
        base = declarative_base()
     base.metadata.bind = engine
@@ -160,15 +234,22 @@ def copy_database_schema(source_connection_string, destination_connection_string
                 dest_session.execute(dest_table.insert(row))
             dest_session.commit()
 
+    # Explicitly close sessions/engines
+    src_session.close()
+    dest_session.close()
+    src_engine.dispose()
+    dest_engine.dispose()
+
 
 class Database:
     def __init__(self, connection_string,
-                 reference_tables=['Publications', 'Telescopes', 'Instruments'],
-                 primary_table='Sources',
-                 primary_table_key='source',
-                 foreign_key='source',
+                 reference_tables=REFERENCE_TABLES,
+                 primary_table=PRIMARY_TABLE,
+                 primary_table_key=PRIMARY_TABLE_KEY,
+                 foreign_key=FOREIGN_KEY,
                  column_type_overrides={},
-                 sqlite_foreign=True):
+                 sqlite_foreign=True,
+                 connection_arguments={}):
         """
         Wrapper for database calls and utility functions
 
@@ -191,14 +272,16 @@ class Database:
             will set the table spectra, column spectrum to be of type TEXT()
         sqlite_foreign : bool
             Flag to enable/disable use of foreign keys with SQLite. Default: True
+        connection_arguments : dict
+            Additional connection arguments, like {'check_same_thread': False}. Default: {}
         """
 
-        self.session, self.base, self.engine = load_connection(connection_string, sqlite_foreign=sqlite_foreign)
+        self.session, self.base, self.engine = load_connection(connection_string, sqlite_foreign=sqlite_foreign,
+                                                               connection_arguments=connection_arguments)
 
         # Convenience methods
         self.query = self.session.query
-
-        self.save_database = self.save_db
+        self.save_db = self.save_database
         self.load_db = self.load_database
 
         # Prep the tables
@@ -211,7 +294,7 @@ class Database:
         self._foreign_key = foreign_key
 
         if len(self.metadata.tables) == 0:
-            print('Database empty. Import schema (eg, from astrodbkit.schema import *) '
+            print('Database empty. Import schema (eg, from astrodbkit.schema_example import *) '
                   'and then run create_database()')
             raise RuntimeError('Create database first.')
 
@@ -226,14 +309,15 @@ class Database:
                 self.metadata.tables[tab].columns[col].type = v
 
     # Generic methods
-    def _handle_format(self, temp, format):
+    @staticmethod
+    def _handle_format(temp, fmt):
         # Internal method to handle SQLAlchemy output and format it
-        if format.lower() in ('astropy', 'table'):
+        if fmt.lower() in ('astropy', 'table'):
             if len(temp) > 0:
                 results = AstropyTable(rows=temp, names=temp[0].keys())
             else:
                 results = AstropyTable(temp)
-        elif format.lower() == 'pandas':
+        elif fmt.lower() == 'pandas':
             results = pd.DataFrame(temp)
         else:
             results = temp
@@ -322,9 +406,10 @@ class Database:
         return data_dict
 
     # Text query methods
+    @deprecated_alias(format='fmt')
     def search_object(self, name, output_table=None, resolve_simbad=False,
-                      table_names={'Sources': 'source', 'Names': 'other_name', 'Sources': 'shortname'},
-                      format='default'):
+                      table_names={'Sources': ['source', 'shortname'], 'Names': ['other_name']},
+                      fmt='default', fuzzy_search=True, verbose=True):
         """
         Query the database for the object specified. By default will return the primary table,
         but this can be specified. Users can also request to resolve the object name via Simbad and query against
@@ -332,17 +417,21 @@ class Database:
 
         Parameters
         ----------
-        name : str
-            Object name to match
+        name : str or list
+            Object name(s) to match
         output_table : str
             Name of table to match. Default: primary table (eg, Sources)
         resolve_simbad : bool
             Get additional names from Simbad. Default: False
         table_names : dict
-            Dictionary of tables to search for name information. Should be of the form table name: column name.
-            Default: {'Sources': 'source', 'Names': 'other_name'}
-        format : str
+            Dictionary of tables to search for name information. Should be of the form table name: column name list.
+            Default: {'Sources': ['source', 'shortname'], 'Names': 'other_name'}
+        fmt : str
             Format to return results in (pandas, astropy/table, default)
+        fuzzy_search : bool
+            Flag to perform partial searches on provided names (default: True)
+        verbose : bool
+            Output some extra messages (default: True)
 
         Returns
         -------
@@ -363,7 +452,8 @@ class Database:
         if resolve_simbad:
             simbad_names = get_simbad_names(name)
             name = list(set(simbad_names + [name]))
-            print(f'Including Simbad names, searching for: {name}')
+            if verbose:
+                print(f'Including Simbad names, searching for: {name}')
 
         # Turn name into a list
         if not isinstance(name, list):
@@ -380,33 +470,39 @@ class Database:
         # This is not really optimized as it does separate DB calls,
         # but is the simpler setup and at our scale is sufficient
         matched_names = []
-        for k, v in table_names.items():
-            filters = [self.metadata.tables[k].columns[v].ilike(f'%{n}%')
-                       for n in name]
+        for k, col_list in table_names.items():
+            for v in col_list:
+                if fuzzy_search:
+                    filters = [self.metadata.tables[k].columns[v].ilike(f'%{n}%')
+                               for n in name]
+                else:
+                    filters = [self.metadata.tables[k].columns[v].ilike(f'{n}')
+                               for n in name]
 
-            # Column to be returned
-            if k == self._primary_table:
-                output_to_match = self.metadata.tables[k].columns[self._primary_table_key]
-            else:
-                output_to_match = self.metadata.tables[k].columns[self._foreign_key]
+                # Column to be returned
+                if k == self._primary_table:
+                    output_to_match = self.metadata.tables[k].columns[self._primary_table_key]
+                else:
+                    output_to_match = self.metadata.tables[k].columns[self._foreign_key]
 
-            temp = self.query(output_to_match).\
-                filter(or_(*filters)).\
-                distinct().\
-                all()
-            matched_names += [s[0] for s in temp]
+                temp = self.query(output_to_match).\
+                    filter(or_(*filters)).\
+                    distinct().\
+                    all()
+                matched_names += [s[0] for s in temp]
 
         # Join the matched sources with the desired table
         temp = self.query(self.metadata.tables[output_table]).\
             filter(self.metadata.tables[output_table].columns[match_column].in_(matched_names)).\
             all()
 
-        results = self._handle_format(temp, format)
+        results = self._handle_format(temp, fmt)
 
         return results
 
     # General query methods
-    def sql_query(self, query, format='default'):
+    @deprecated_alias(format='fmt')
+    def sql_query(self, query, fmt='default'):
         """
         Wrapper for a direct SQL query.
 
@@ -414,7 +510,7 @@ class Database:
         ----------
         query : str
             Query to be performed
-        format : str
+        fmt : str
             Format in which to return the results (pandas, astropy/table, default)
 
         Returns
@@ -424,7 +520,7 @@ class Database:
 
         temp = self.engine.execute(query).fetchall()
 
-        return self._handle_format(temp, format)
+        return self._handle_format(temp, fmt)
 
     # Object output methods
     def save_json(self, name, directory):
@@ -452,7 +548,7 @@ class Database:
         with open(os.path.join(directory, filename), 'w') as f:
             f.write(json.dumps(data, indent=4, default=json_serializer))
 
-    def save_db(self, directory, clear_first=True):
+    def save_database(self, directory, clear_first=True):
         """
         Output contents of the database into the specified directory as JSON files.
         Source objects have individual JSON files with all data for that object.
@@ -473,6 +569,10 @@ class Database:
 
         # Output reference tables
         for table in self._reference_tables:
+            # Skip reference tables that are not actually in the database
+            if table not in self.metadata.tables.keys():
+                continue
+
             results = self.session.query(self.metadata.tables[table]).all()
             data = [row._asdict() for row in results]
             filename = table + '.json'
@@ -516,7 +616,7 @@ class Database:
         """
 
         with open(filename, 'r') as f:
-            data = json.load(f)
+            data = json.load(f, object_hook=datetime_json_parser)
 
         # Loop through the dictionary, adding data to the database.
         # Ensure that Sources is added first
